<?php

/**
 * @file
 * image_target_question.class.inc
 * Question type, enabling the creation of image_target_question questions
 */

/**
 * Extension of QuizQuestion.
 */
class ImageTargetQuestion extends QuizQuestion {

  /**
   * Run check_markup() on the field of the specified choice alternative
   *
   * @param int $alternative_index
   *   The index of the alternative in the alternatives array.
   * @param string $field
   *   The name of the field we want to check markup on
   * @param boolean $check_user_access
   *   Whether or not to check for user access to the filter
   *
   * @return string
   *   HTML markup
   */
  public function checkMarkup($alternative_index, $field, $check_user_access = TRUE) {
    $alternative = $this->node->alternatives[$alternative_index];
    return check_markup($alternative[$field], $alternative[$field . '_format'], $check_user_access);
  }

  /**
   * Implementation of save
   * Stores the question in the database.
   *
   * @param boolean $is_new
   *   TRUE if - if the node is a new node
   *
   * @see quiz/question_types/quiz_question/QuizQuestion#save()
   */
  public function saveNodeProperties($is_new = FALSE) {
    $is_new = $is_new || $this->node->revision == 1;

    $file = image_target_question_save_image('alternatives');

    if ($file) {
      $this->node->fid = $file->fid;
      $this->node->width = $file->width;
      $this->node->height = $file->height;
    }

    if (!isset($this->node->feedback_format)) {
      $this->node->feedback_format = FILTER_FORMAT_DEFAULT;
    }

    if ($is_new) {

      if (!isset($this->node->fid)) {
        watchdog('image_target_question', 'Could not find any image attached to the node on save %nid', array('%nid' => $this->node->nid), WATCHDOG_ERROR);
        $this->node->fid = 0;
      }

      $sql = 'INSERT INTO {quiz_image_target_question_properties}
      (nid, vid, fid, width, height, any_region, feedback, feedback_format)
      VALUES (%d, %d, %d, %d, %d, %d, \'%s\', %d)';
      db_query($sql, $this->node->nid, $this->node->vid, $this->node->fid, $this->node->width, $this->node->height, $this->node->alternatives['any_region'], $this->node->feedback, $this->node->feedback_format);
      for ($i = 1; isset($this->node->alternatives['targets'][$i]); $i++) {
        if (drupal_strlen($this->node->alternatives['targets'][$i]['targettext']) > 0) {
          $this->insertAlternative($i);
        }
      }
    }
    else {

      $sql = 'UPDATE {quiz_image_target_question_properties}
        SET fid = %d, width = %d, height = %d, any_region = %d, feedback = \'%s\', feedback_format = %d
        WHERE nid = %d AND vid = %d';
      db_query($sql, $this->node->fid, $this->node->width, $this->node->height, $this->node->alternatives['any_region'], $this->node->feedback, $this->node->feedback_format, $this->node->nid, $this->node->vid);

      // We fetch ids for the existing answers belonging to this question
      // We need to figure out if an existing alternative has been changed
      // or deleted.
      $sql = 'SELECT id FROM {quiz_image_target_question_target}
              WHERE question_nid = %d AND question_vid = %d';
      $res = db_query($sql, $this->node->nid, $this->node->vid);

      // Start by assuming that all existing alternatives needs to be deleted.
      $ids_to_delete = array();
      while ($res_o = db_fetch_object($res)) {
        $ids_to_delete[] = $res_o->id;
      }

      for ($i = 1; isset($this->node->alternatives['targets'][$i]); $i++) {
        $short = $this->node->alternatives['targets'][$i];
        if (drupal_strlen($this->node->alternatives['targets'][$i]['targettext']) > 0) {
          if (!is_numeric($short['id'])) {
            // If new alternative.
            $this->insertAlternative($i);
          }
          else {
            // If existing alternative.
            $this->updateAlternative($i);
            // Make sure this alternative isn't deleted.
            $key = array_search($short['id'], $ids_to_delete);
            $ids_to_delete[$key] = FALSE;
          }
        }
      }
      foreach ($ids_to_delete as $id_to_delete) {
        if ($id_to_delete) {
          db_query('DELETE FROM {quiz_image_target_question_target} WHERE id = %d', $id_to_delete);
        }
      }
    }
  }

  /**
   * Helper function. Saves new alternatives
   *
   * @param int $i
   *   The alternative index
   */
  public function insertAlternative($i) {
    $sql = 'INSERT INTO {quiz_image_target_question_target}
            (targettext, top, leftpx, width, height, score_if_correct,
            score_if_not_correct, question_nid, question_vid)
            VALUES(\'%s\', %d, %d, %d, %d, %d, %d, %d, %d)';
    $short = $this->node->alternatives['targets'][$i];
    $position = explode(',', $short['region']);
    db_query($sql, $short['targettext'], $position[0], $position[1], $position[2], $position[3], $short['score_if_correct'], $short['score_if_not_correct'], $this->node->nid, $this->node->vid);
  }

  /**
   * Helper function. Updates existing alternatives
   *
   * @param int $i
   *   The alternative index
   */
  public function updateAlternative($i) {
    $sql = 'UPDATE {quiz_image_target_question_target}
            SET targettext = \'%s\', top = %d, leftpx = %d, width = %d, height = %d,
            score_if_correct = %d, score_if_not_correct = %d
            WHERE id = %d AND question_nid = %d AND question_vid = %d';
    $short = $this->node->alternatives['targets'][$i];
    $position = explode(',', $short['region']);
    db_query($sql, $short['targettext'], $position[0], $position[1], $position[2], $position[3], $short['score_if_correct'], $short['score_if_not_correct'], $short['id'], $this->node->nid, $this->node->vid);
  }

  /**
   * Implementation of validate
   */
  public function validateNode(array &$form) {
    if ($this->node->revision == 1) {
      // TODO: check a file was uploaded if new.
    }


    for ($i = 1; isset($this->node->alternatives['targets'][$i]); $i++) {
      $short = $this->node->alternatives['targets'][$i];

      // TODO: check the positional elements are correct and sane.
    }
  }

  /**
   * Implementation of delete
   * @see QuizQuestion#delete()
   */
  public function delete($only_this_version = FALSE) {
    $delete_properties = 'DELETE FROM {quiz_image_target_question_properties} WHERE nid = %d';
    $delete_targets = 'DELETE FROM {quiz_image_target_question_target} WHERE question_nid = %d';
    $delete_multi_results = '
      DELETE FROM {quiz_image_target_question_user_answer_targets}
      USING {quiz_image_target_question_user_answer_targets}
      INNER JOIN {quiz_image_target_question_user_answers}
      ON {quiz_image_target_question_user_answer_targets}.user_answer_id = {quiz_image_target_question_user_answers}.id
      WHERE {quiz_image_target_question_user_answers}.question_nid = %d' . ($only_this_version ? ' AND question_vid = %d' : '');
    $delete_results = 'DELETE FROM {quiz_image_target_question_user_answers} WHERE question_nid = %d';
    if ($only_this_version) {
      $delete_properties .= ' AND vid = %d';
      $delete_targets .= ' AND question_vid = %d';
      $delete_results .= ' AND question_vid = %d';
    }
    db_query($delete_properties, $this->node->nid, $this->node->vid);
    db_query($delete_targets, $this->node->nid, $this->node->vid);
    db_query($delete_multi_results, $this->node->nid, $this->node->vid);
    db_query($delete_results, $this->node->nid, $this->node->vid);

    parent::delete($only_this_version);
  }

  /**
   * Implementation of getNodeProperties
   * @see QuizQuestion#getNodeProperties()
   */
  public function getNodeProperties() {
    if (isset($this->nodeProperties)) {
      return $this->nodeProperties;
    }
    $props = parent::getNodeProperties();

    // Load the properties.
    $sql = 'SELECT * FROM {quiz_image_target_question_properties} WHERE nid = %d AND vid = %d';
    $res = db_query($sql, $this->node->nid, $this->node->vid);
    $res_a = db_fetch_array($res);
    if (is_array($res_a)) {
      $props = array_merge($props, $res_a);
    }

    if (is_null($this->node->any_region)) {
      $this->node->any_region = 0;
    }

    // Load the targets.
    $sql = 'SELECT *
      FROM {quiz_image_target_question_target}
      WHERE question_nid = %d AND question_vid = %d
      ORDER BY id';
    $res = db_query($sql, $this->node->nid, $this->node->vid);
    $props['alternatives']['targets'] = array();

    $i = 1;
    while ($res_arr = db_fetch_array($res)) {
      $props['alternatives']['targets'][$i] = $res_arr;
      $i++;
    }

    $this->nodeProperties = $props;

    return $props;
  }

  /**
   * Implementation of getNodeView
   * @see QuizQuestion#getNodeView()
   */
  public function getNodeView() {
    $content = parent::getNodeView();

    if (user_access('edit any quiz') && !isset($this->node->alternatives['targets'])) {
      drupal_set_message(t('There are no targets set on this image'));
      $content['answers'] = array(
        '#value' => '',
        '#weight' => 2,
      );
    }
    else {
      $content['answers'] = array(
        '#value' => theme('image_target_question_answer_node_view', $this->node->alternatives['targets'], $this->viewCanRevealCorrect(), $this->node->fid, $this->node->any_region),
        '#weight' => 2,
      );
    }

    return $content;
  }

  /**
   * Generates the question form.
   *
   * This is called whenever a question is rendered, either
   * to an administrator or to a quiz taker.
   */
  public function getAnsweringForm(array $form_state, $rid) {
    $form = parent::getAnsweringForm($form_state, $rid);

    if (user_access('edit any quiz') && empty($this->node->alternatives['targets'])) {
      drupal_set_message(t('You can now set some targets on this image'));
      drupal_goto('node/' . $this->node->nid . '/edit');
    }

    $form['#theme'] = 'image_target_question_answering_form';

    $form['image_target_question_image'] = $this->generateImageMarkupFormElement();

    $res = '';
    if (isset($rid)) {
      // This question has already been answered. We load the answer.
      $response = new ImageTargetResponse($rid, $this->node);
      $response = $response->getResponse();
      $res = array();
      foreach ($response as $id => $r) {
        $res[] = $r['top'] . ',' . $r['leftpx'];
      }
      $res = implode(':', $res);
    }


    $form['tries[answer]'] = array(
        '#type' => user_access('access devel information') ? 'hidden' : 'hidden',
        '#default_value' => isset($res) ? $res : 'here',
        '#size' => 10,
        '#attributes' => array('class' => 'image_target_question-target'),
      );

    $form['any_region'] = array(
      '#type' => 'value',
      '#value' => $this->node->any_region,
    );

    $form['targets'] = array(
      '#type' => 'fieldset',
      '#title' => t('Answer'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
      '#weight' => -4,
      '#tree' => TRUE,
    );

    for ($i = 1; isset($this->node->alternatives['targets'][$i]); $i++) {
      $short = $this->node->alternatives['targets'][$i];

      $form['targets'][$i]['id'] = array(
        '#type' => 'value',
        '#value' => $short['id'],
      );

      $form['targets'][$i]['targettext'] = array(
        '#type' => 'markup',
        '#value' => check_plain($short['targettext']),
      );

      $form['targets'][$i]['selected_target'] = array(
        '#type' => 'button',
        '#value' => t('Set target !i', array('!i' => $i)),
        '#attributes' => array('class' => 'image_target_question-select-region'),
        );
    }

    return $form;
  }

  /**
   * Implementation of getCreationForm
   * @see QuizQuestion#getCreationForm()
   */
  public function getCreationForm(array $form_state = NULL) {

    // Ajax may have added an image.
    if (isset($form_state['values']['fid'])) {
      $this->node->fid = $form_state['values']['fid'];
      $this->node->width = $form_state['values']['width'];
      $this->node->height = $form_state['values']['height'];
    }

    $form = array();
    $form['#attributes'] = array('enctype' => "multipart/form-data");

    $type = node_get_types('type', $this->node);

    // We add #action to the form because of the use of ajax.
    $options = array();
    $get = $_GET;
    unset($get['q']);
    if (!empty($get)) {
      $options['query'] = $get;
    }

    $action = url('node/add/image-target-question', $options);
    if (isset($this->node->nid)) {
      $action = url('node/' . $this->node->nid . '/edit', $options);
    }
    $form['#action'] = $action;

    if (isset($this->node->fid)) {

      $form['fid'] = array(
        '#type' => 'value',
        '#value' => $this->node->fid,
      );

      $form['width'] = array(
        '#type' => 'value',
        '#value' => $this->node->width,
      );

      $form['height'] = array(
        '#type' => 'value',
        '#value' => $this->node->height,
      );
    }

    $form['alternatives'] = array(
      '#type' => 'fieldset',
      '#title' => t('Answer'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
      '#weight' => -4,
      '#tree' => TRUE,
      '#prefix' => '<div id="image_target_question_alternatives">',
      '#suffix' => '</div>',
    );

    $form['alternatives']['any_region'] = array(
      '#type' => 'checkbox',
      '#title' => t('Any target in any region'),
      '#description' => t('Tick this if the user gets the "score if correct" score if they put a target in any region'),
      '#default_value' => isset($this->node->any_region) ? $this->node->any_region : 0,
    );

    $form['alternatives']['upload'] = array(
      '#type' => 'file',
      '#title' => t('Attach a new image file'),
      '#description' => t('After adding the image, save the question then come back to set the targets.'),
    );

    $form['alternatives']['upload_now'] = array(
      '#type' => 'submit',
      '#value' => 'Upload',
      '#submit' => array('image_target_question_upload_image'),
      '#ahah' => array(
        'path' => 'node/add/image_target_question/upload_image_ahah',
        'wrapper' => 'image_target_question_alternatives',
        'effect' => 'fade',
        'method' => 'replace',
        'progress' => array('type' => 'bar', 'message' => t('Please wait...')),
      ),
    );

    $form['alternatives']['image_target_question_image'] = $this->generateImageMarkupFormElement();

    $form['alternatives']['#theme'][] = 'image_target_question_creation_form';

    // The choice_count might be stored in the form_state after ajax callback.
    if (isset($form_state['choice_count'])) {
      $choice_count = $form_state['choice_count'];
    }
    else {
      $choice_count = max(variable_get('image_target_question_def_num_of_alts', 10), isset($this->node->alternatives['targets']) ? count($this->node->alternatives['targets']) : 0);
    }

    // We only add the targets if there is an image set.
    if (isset($this->node->fid)) {

      for ($i = 1; $i <= $choice_count; $i++) {
        $short = isset($this->node->alternatives['targets'][$i]) ? $this->node->alternatives['targets'][$i] : NULL;

        $form['alternatives']['targets'][$i]['id'] = array(
          '#type' => 'value',
          '#value' => $short['id'],
        );

        $form['alternatives']['targets'][$i]["targettext"] = array(
          '#type' => 'textfield',
          '#default_value' => $short['targettext'],
        );

        $form['alternatives']['targets'][$i]['selected_region'] = array(
          '#type' => 'button',
          '#value' => t('Set region !i', array('!i' => $i)),
          '#attributes' => array('class' => 'image_target_question-select-region'),
        );

        $form['alternatives']['targets'][$i]['clear_region'] = array(
          '#type' => 'button',
          '#value' => t('Clear region !i', array('!i' => $i)),
          '#attributes' => array('class' => 'image_target_question-clear-region'),
        );

        $form['alternatives']['targets'][$i]['region'] = array(
          '#type' => user_access('access devel information') ? 'hidden' : 'hidden',
          '#default_value' => ( isset($short) ? "{$short['top']},{$short['leftpx']},{$short['width']},{$short['height']}" : ''),
          '#size' => 10,
          '#attributes' => array('class' => 'image_target_question-region'),
        );

        $form['alternatives']['targets'][$i]['score_if_correct'] = array(
          '#type' => 'textfield',
          '#size' => 2,
          '#maxlength' => 2,
          '#default_value' => isset($short['score_if_correct']) ? $short['score_if_correct'] : 1,
        );

        $form['alternatives']['targets'][$i]['score_if_not_correct'] = array(
          '#type' => 'textfield',
          '#size' => 4,
          '#maxlength' => 4,
          '#default_value' => isset($short['score_if_not_correct']) ? $short['score_if_not_correct'] : 0,
        );
      }
    }

    // TODO : Add ahah based button to add more regions.
    $form['feedback'] = array(
      '#type' => 'textarea',
      '#title' => t('Feedback'),
      '#description' => t('Enter the feedback the users will see on the report at the end of the quiz'),
      '#default_value' => $this->node->feedback,
    );

    $form['feedback_format'] = filter_form($this->node->feedback_format, NULL, array('feedback_format'));

    return $form;
  }

  /**
   * Helper function to generate the image in the form
   */
  public function generateImageMarkupFormElement() {
    $default_settings = $this->getDefaultAltSettings();

    $fileimg = '';
    $height = '';
    if ($default_settings['fid']) {
      $file = image_target_question_file_load($default_settings['fid']);
      if ($file) {
        $imgurl = url($file->filepath);
        if (image_target_question_get_imagecache_preset()) {
          $imgurl = imagecache_create_url(image_target_question_get_imagecache_preset()->presetname, $file->filepath);
        }
        $fileimg = '<img src="' . $imgurl . '" style="position: absolute;" />';
        $height = ' height: ' . (intval($default_settings['height']) + 25) . 'px;';
      }
    }

    return array(
      '#type' => 'markup',
      '#value' => "<div id=\"image_target_question_image\" style=\"position: relative;{$height}\">{$fileimg}</div>",
    );
  }

  /**
   * Helper function provding the default settings for the creation form.
   *
   * @return array
   *   Array with the default settings
   */
  public function getDefaultAltSettings() {
    return array(
      'fid' => isset($this->node->fid) ? $this->node->fid : 0,
      'width' => isset($this->node->width) ? $this->node->width : 0,
      'height' => isset($this->node->fid) ? image_target_question_get_image_height($this->node->fid) : 0,
      'feedback' => isset($this->node->feedback) ? $this->node->feedback : '',
      'feedback_format' => isset($this->node->feedback_format) ? $this->node->feedback_format : FILTER_FORMAT_DEFAULT,
    );
  }

  /**
   * Implementation of getMaximumScore.
   * Calculate the maximum score for this question
   * @return int
   *   The maximum score possible on this question
   * @see QuizQuestion#getMaximumScore()
   */
  public function getMaximumScore() {
    $max = 0;
    for ($i = 1; isset($this->node->alternatives['targets'][$i]); $i++) {
      if (drupal_strlen($this->node->alternatives['targets'][$i]['targettext']) > 0) {
        $short = $this->node->alternatives['targets'][$i];
        $max += max($short['score_if_correct'], $short['score_if_not_correct']);
      }
    }
    return max($max, 1);
  }

}

/**
 * Extension of QuizQuestionResponse
 */
class ImageTargetResponse extends QuizQuestionResponse {
  /**
   * ID of the answers.
   */
  public $user_answers;

  /**
   * Constructor
   */
  public function __construct($result_id, stdClass $question_node, $tries = NULL) {
    parent::__construct($result_id, $question_node, $tries);
    $this->user_answers = array();

    if (is_array($tries)) {
      $answer = is_array($tries['answer']) ? $tries['answer']['answer'] : $tries['answer'];
      $targets = explode(":", $answer);
      $i = 1;
      foreach ($targets as $target) {
        $coords = explode(',', $target);
        $this->user_answers[$i] = array('top' => $coords[0], 'leftpx' => $coords[1], 'target_id' => $question_node->alternatives['targets'][$i]['id']);
        $i++;
      }
    }
    else {
      $sql = 'SELECT target_id,top,leftpx
        FROM {quiz_image_target_question_user_answers} ua
        LEFT OUTER JOIN {quiz_image_target_question_user_answer_targets} uam ON (uam.user_answer_id = ua.id)
        WHERE ua.result_id = %d AND ua.question_nid = %d AND ua.question_vid = %d';
      $res = db_query($sql, $result_id, $this->question->nid, $this->question->vid);
      $i = 1;
      while ($res_o = db_fetch_array($res)) {
        $this->user_answers[$i] = $res_o;
        $i++;
      }
    }

    // Make sure there are as many answers as regions.
    foreach ($this->question->alternatives['targets'] as $id => $region) {
      if (!isset($this->user_answers[$id]) || !is_numeric($this->user_answers[$id]['top']) || !is_numeric($this->user_answers[$id]['leftpx'])) {
        $this->user_answers[$id] = array('top' => -1, 'leftpx' => -1, 'target_id' => $question_node->alternatives['targets'][$id]['id']);
      }
    }
  }

  /**
   * Implementation of isValid
   *
   * @see QuizQuestionResponse#isValid()
   */
  public function isValid() {
    if (empty($this->user_answers)) {
      return t('You place every target');
    }

    foreach ($this->user_answers as $coords) {
      if (count($coords) != 2 || !is_numeric($coords[0]) || !is_numeric($coords[1])) {
        // TODO: Should they place every target?
      }
    }
    return TRUE;
  }

  /**
   * Implementation of save
   *
   * @see QuizQuestionResponse#save()
   */
  public function save() {
    $sql = "INSERT INTO {quiz_image_target_question_user_answers}
    (result_id, question_vid, question_nid)
    VALUES (%d, %d, %d)";
    db_query($sql, $this->rid, $this->question->vid, $this->question->nid);
    $user_answer_id = db_last_insert_id('{quiz_image_target_question_user_answers}', 'id');
    for ($i = 1; $i <= count($this->user_answers); $i++) {
      $sql = 'INSERT INTO {quiz_image_target_question_user_answer_targets}
      (user_answer_id, target_id, top, leftpx)
      VALUES(%d, %d, %d, %d)';
      db_query($sql, $user_answer_id, $this->user_answers[$i]['target_id'], $this->user_answers[$i]['top'], $this->user_answers[$i]['leftpx']);
    }
  }

  /**
   * Implementation of delete
   *
   * @see QuizQuestionResponse#delete()
   */
  public function delete() {
    $sql = 'DELETE FROM {quiz_image_target_question_user_answer_targets}
    USING {quiz_image_target_question_user_answer_targets}
    INNER JOIN {quiz_image_target_question_user_answers}
    ON {quiz_image_target_question_user_answer_targets}.user_answer_id = {quiz_image_target_question_user_answers}.id
    WHERE {quiz_image_target_question_user_answers}.question_nid = %d
    AND {quiz_image_target_question_user_answers}.question_vid = %d
    AND {quiz_image_target_question_user_answers}.result_id = %d';
    db_query($sql, $this->question->nid, $this->question->vid, $this->rid);
    $sql = 'DELETE FROM {quiz_image_target_question_user_answers}
    WHERE result_id = %d AND question_nid = %d AND question_vid = %d';
    db_query($sql, $this->rid, $this->question->nid, $this->question->vid);
  }

  /**
   * Implementation of score
   * @see QuizQuestionResponse#score()
   */
  public function score() {
    $score = 0;
    foreach ($this->question->alternatives['targets'] as $key => $region) {
      $in = empty($this->question->any_region) ?
        $this->checkTargetInRegion($this->user_answers[$key], $region) :
        $this->checkAnyTargetInRegion(&$this->question->alternatives['targets'][$key]);
      $score += $in ? $region['score_if_correct'] : $region['score_if_not_correct'];
    }
    return $score;
  }

  /**
   * Given a target and region, checks that the target is in the region
   *
   * @param array $target
   *   Values are in px ['top', 'leftpx'].
   * @param array $region
   *   Values in px ['top', 'leftpx', 'width', 'height']
   *
   * @return boolean
   *   TRUE if target is inside the region, FALSE if outside
   */
  public function checkTargetInRegion($target, $region) {
    if ($this->regionIsNonRegion($region)) {
      // In this case, the user scores for getting the target outside
      // the bounds of the image.
      return !$this->checkTargetInImage($target);
    }
    else {
      // In this case, the user scores for getting the target in the bounds
      // of the region.
      return  ($target['leftpx'] >= $region['leftpx'] && $target['leftpx'] <= ($region['leftpx'] + $region['width'])
        && $target['top'] >= $region['top'] && $target['top'] <= ($region['top'] + $region['height']));
    }
  }

  /**
   * Determine if a region is a non-region, i.e. one which is not on the image.
   *
   * @return boolean
   *   TRUE if it is a non-region.
   */
  public function regionIsNonRegion($region) {
    return ($region['width'] == 0 && $region['height'] == 0);
  }

  /**
   * Check if any of the users targets were in the region.
   *
   * @return boolean
   *   TRUE if any target was in the specified region.
   */
  public function checkAnyTargetInRegion(&$region) {
    if ($this->regionIsNonRegion($region)) {
      // In the case of a non-region, see if a target is
      // outside the image.
      if (isset($region['answer_ref'])) {
        // We have linked an answer to this region permenantly.
        // It must be outside the bounds of the image.
        return $region['answer_ref'] > 0;
      }

      // Find an answer outside the bounds of the image.
      foreach ($this->user_answers as $tkey => $target) {
        if (!isset($target['non_region_ref']) && !$this->checktargetinimage($target)) {
          // Assign that target permenantly
          // to this region as correct.
          $this->user_answers[$tkey]['non_region_ref'] = $region['id'];
          $region['answer_ref'] = $tkey;
          return TRUE;
        }
      }

      // There were no targets showing this region is correct.
      $region['answer_ref'] = 0;
    }
    else {
      foreach ($this->user_answers as $target) {
        if ($this->checkTargetInRegion($target, $region)) {
          return TRUE;
        }
      }
    }
    return FALSE;
  }

  /**
   * Check the user placed the target into the bounds of the image
   *
   * @return boolean
   *   TRUE if they did and FALSE if they did not.
   */
  public function checkTargetInImage($target) {
    return !(empty($target) || $target['leftpx'] <= 0 || $target['leftpx'] >= $this->question->width
      || $target['top'] <= 0 || $target['top'] >= $this->question->height);
  }

  /**
   * Implementation of getResponse
   * @see QuizQuestionResponse#getResponse()
   */
  public function getResponse() {
    return $this->user_answers;
  }

  /**
   * Implementation of getReportFormResponse
   *
   * @see getReportFormResponse($showpoints, $showfeedback, $allow_scoring)
   */
  public function getReportFormResponse($showcorrect = TRUE, $showfeedback = TRUE, $allow_scoring = FALSE) {
    $feedback = $this->checkMarkup($this->question->feedback, $this->question->feedback_format);

    $answers = array();
    foreach ($this->question->alternatives['targets'] as $i => $region) {
      if (drupal_strlen(check_plain($region['targettext'])) > 0) {
        $in = empty($this->question->any_region) ?
          $this->checkTargetInRegion($this->user_answers[$i], $region) :
          $this->checkAnyTargetInRegion($this->question->alternatives['targets'][$i]);

        $answers[$i]['targettext'] = $region['targettext'] . ($this->regionIsNonRegion($region) ? ' (<em>' . t('This item was not on the image') . '</em>)' : '');
        $answers[$i]['score'] = $in ? $region['score_if_correct'] : $region['score_if_not_correct'];
        $answers[$i]['correct'] = $answers[$i]['score'] == max($region['score_if_correct'], $region['score_if_not_correct']);

        if (!$this->regionIsNonRegion($region) || !$this->question->any_region) {
          $answers[$i]['region'] = $region;
        }
        $answers[$i]['target'] = $this->checkTargetInImage($this->user_answers[$i]) ? $this->user_answers[$i] : FALSE;
      }
    }

    return array(
      '#type' => 'markup',
      '#value' => theme('image_target_question_response', $this->question->fid, $answers, $feedback, $showcorrect, $showfeedback, $this->question->any_region),
    );
  }

  /**
   * Run check_markup() on the field of the specified choice alternative
   *
   * @param string $alternative
   *   text to be checked
   * @param int $format
   *   The input format to be used
   * @param boolean $check_user_access
   *   Whether or not we are to check the users access to the chosen format
   *
   * @return string
   *   The clean HTML markup
   */
  public function checkMarkup($alternative, $format, $check_user_access = FALSE) {
    if (drupal_strlen($alternative) == 0) {
      return '';
    }
    return check_markup($alternative, $format, $check_user_access);
  }
}
